@@ -16,8 +16,7 @@ import jax
 import dill
 import re
 import os
-
-
+globalk = False
 class Model():
     def __init__(self, dist: str, left=4, mask_size=0, model_name='window', param_mode='window'):
         self.dist = dist
@@ -76,6 +75,17 @@ class Model():
         res, c = np.unique(res, axis=0, return_counts=True)
         res = np.append(res, c.reshape(-1, 1), axis=1)
         return res
+    
+    def minimize_scalar(self, f, xatol=1e-8, steps=10):
+        if steps:
+            ps = list(np.linspace(0.0001, 0.9999, steps))
+            i = np.argmin(list(map(f, ps))) + 1
+            ps = [0.0] + ps + [1.0]
+            b = ps[i - 1], ps[i + 1]
+        else:
+            b = (0.0, 1.0)
+        return minimize_scalar(f, bounds=b, method='bounded', options={'xatol': xatol})
+        
 
     def fit(self, data: np.ndarray, params: dict, compute_var=True, sandwich=True, n_bootstrap=100):
         name = self.model_name
@@ -98,12 +108,14 @@ class Model():
                 k = ps['mu_k'] + ps['b_k'] * np.log(fixed)
             else:
                 k = np.zeros(len(r))
+        if globalk:
+            print(data, r, k, w)
         data, r, k, w = self.update_mask(data, r, k, w)
         mask = self.mask
         f = partial(self.negloglik, r=r, k=k, data=data, w=w, mask=mask)
         grad_w = partial(self.grad, r=r, k=k, data=data, w=w, mask=mask)
         fim = partial(self.fim, r=r, k=k, data=data, w=w, mask=mask)
-        res = minimize_scalar(f, bounds=(0.0, 1.0), method='bounded')
+        res = self.minimize_scalar(f)
         x = res.x
         bs = list()
         for i in range(n_bootstrap):
@@ -112,7 +124,7 @@ class Model():
             data, r_, k_, w_ = self.update_mask(data, r_, k_, w_)
             mask = self.mask
             f = partial(self.negloglik, r=r_, k=k_, data=data, w=w_, mask=mask)
-            res_ = minimize_scalar(f, bounds=(0.0, 1.0), method='bounded')
+            res_ = self.minimize_scalar(f)
             if res_.success:
                 bs.append(res_.x) 
 
@@ -203,7 +215,7 @@ def get_closest_param(params: dict, slc: float, model_name: str, compute_line=Fa
 
 def lrt_test(counts: Tuple[tuple, np.ndarray, np.ndarray, np.ndarray],
         inst_params: dict, params: dict, skip_failures=False, max_sz=None, bad=1.0,
-        param_mode='window'):
+        param_mode='window', n_bootstrap=0):
     if not hasattr(lrt_test, '_cache'):
         lrt_test._cache = dict()
     snv, counts_a, counts_b, counts = counts
@@ -219,11 +231,11 @@ def lrt_test(counts: Tuple[tuple, np.ndarray, np.ndarray, np.ndarray],
         if allele == 'alt':
             counts_a = counts_a[:, (1, 0, 2)]; counts_b = counts_b[:, (1, 0, 2)]; counts = counts[:, (1, 0, 2)]
         try:
-            ab_r, ab_logl = model.fit(counts, params[allele], False)
+            ab_r, ab_logl = model.fit(counts, params[allele], False, n_bootstrap=n_bootstrap)
             ab_p = ab_r.x
-            a_r, a_logl = model.fit(counts_a, params[allele], False)
+            a_r, a_logl = model.fit(counts_a, params[allele], False, n_bootstrap=n_bootstrap)
             a_p = a_r.x
-            b_r, b_logl = model.fit(counts_b, params[allele], False)
+            b_r, b_logl = model.fit(counts_b, params[allele], False, n_bootstrap=n_bootstrap)
             b_p = b_r.x
             success = a_r.success & b_r.success & ab_r.success
             if not success and skip_failures:
@@ -232,7 +244,7 @@ def lrt_test(counts: Tuple[tuple, np.ndarray, np.ndarray, np.ndarray],
             if skip_failures:
                 return None, snv
             res.append((float('nan'), float('nan'), float('nan'), float('nan')))
-        lrt = -2 * (ab_logl - (a_logl + b_logl))
+        lrt = 2 * (ab_logl - (a_logl + b_logl))
         pval = chi2.sf(lrt, 1)
         res.append((pval, ab_p, a_p, b_p))
     n_a, n_b = counts_a[:, -1].sum(), counts_b[:, -1].sum()
@@ -274,11 +286,22 @@ def wald_test(counts: Tuple[tuple, np.ndarray, np.ndarray, np.ndarray],
         if allele == 'alt':
             counts_a = counts_a[:, (1, 0, 2)]; counts_b = counts_b[:, (1, 0, 2)]; counts = counts[:, (1, 0, 2)]
         try:
+            global globalk
+            if snv == ('chr12', 14882146):
+                globalk = True
             a_r, a_var = model.fit(counts_a, params[allele], sandwich=robust_se, n_bootstrap=n_bootstrap)
+            a_var = np.clip(a_var, 0.0, np.inf)
             a_p = a_r.x
             b_r, b_var = model.fit(counts_b, params[allele], sandwich=robust_se, n_bootstrap=n_bootstrap)
+            b_var = np.clip(b_var, 0.0, np.inf)
             b_p = b_r.x
-            correct = (a_var >= 0) & (b_var >= 0) & np.isfinite(a_var) & np.isfinite(b_var)
+            correct = (a_var >= 0) & (b_var >= 0) & (a_var + b_var > 0) & np.isfinite(a_var) & np.isfinite(b_var)
+            if globalk:
+                print(a_r)
+                print(a_var ** 0.5)
+                print(b_r)
+                print(b_var ** 0.5)
+            globalk = False
             if logit_transform:
                 a_p, a_var = transform_p(a_p, a_var)
                 b_p, b_var = transform_p(b_p, b_var)
@@ -350,7 +373,7 @@ def differential_test(name: str, group_a: List[str], group_b: List[str], mode='w
         cols = ['ref_pval', 'ref_p_ab', 'ref_p_a', 'ref_p_b', 
                 'alt_pval', 'alt_p_ab', 'alt_p_a', 'alt_p_b']
         test_fun = partial(lrt_test, inst_params=inst_params, params=params, skip_failures=False, bad=bad,
-                           param_mode=param_mode)
+                           param_mode=param_mode, n_bootstrap=n_bootstrap)
     else:
         cols = ['ref_pval', 'ref_p_a', 'ref_p_b', 'ref_std_a', 'ref_std_b',
                 'alt_pval', 'alt_p_a', 'alt_p_b', 'alt_std_a', 'alt_std_b']
